'''
[MNIST手写数据集，PyTorch实现]

[环境要求]
- torch
- torchvision

[数据]
在 datasets.MNIST 中设置 download=True 即可自动下载
'''

import torch
from torchvision import datasets, transforms

# 读入MNIST训练和测试数据
train_set = datasets.MNIST('./MNIST_train', download=True, train=True, transform=transforms.ToTensor())
test_set = datasets.MNIST('./MNIST_test', download=True, train=False, transform=transforms.ToTensor())

train_loader = torch.utils.data.DataLoader(train_set, batch_size=64, shuffle=True)
test_loader = torch.utils.data.DataLoader(test_set, batch_size=64, shuffle=False)

# 定义两层全连接层网络
net = torch.nn.Sequential(torch.nn.Linear(28*28, 256),
						  torch.nn.ReLU(),
						  torch.nn.Linear(256, 10))

# 定义损失函数
criterion = torch.nn.CrossEntropyLoss()

# 定义优化器
optimizer = torch.optim.Adam(net.parameters(), lr=2e-4)

# 开始训练
for step, data in enumerate(train_loader):
	# 读取数据并处理
	x, y = data
	x = x.view(-1,784)
	y = y.long()

	# 模型前传
	output = net(x)

	# 计算损失函数
	loss = criterion(output, y)

	# 梯度反传和模型参数更新
	optimizer.zero_grad()
	loss.backward()
	optimizer.step()

	print("[{}|{}] Loss:{}".format(step, len(train_loader), loss.item()))

# 开始测试
correct_num = 0.
total_num = 0.

for data in test_loader:
	# 读取数据并处理
	x, y = data
	x = x.view(-1,784)
	y = y.long()

	# 模型前传并计算分类准确率
	with torch.no_grad():
		_, pred = net(x).max(1)
		correct_num += (pred == y).sum().item()
		total_num += len(y)

# 打印准确率
print('Test Accuracy:', correct_num/total_num)
